.page.auth img.logo {
    border-radius: 3px;
    opacity: .9;
    box-shadow: 0 1px 1px rgba(0,0,0,.5);
    display: none !important;
}

.page.auth img.logo.visible {
    display: inline-block !important;
}

#header .logo .text {
    display: none;
}

/* Logo image (main) */
#header .logo {
    width: 160px;
    height: 30px;
    float: left;

    background: url("http://www.ponvia.com/wp-content/themes/ponvia/images/logo-new.jpg");
    background-size: contain;
    background-repeat: no-repeat;
}

/* Accent colors */
#tab-headers .active {
    background: #feba1b !important;
}

.main-sections-category .active {
    background: #feba1b !important;
    color: rgba(0,0,0,.75) !important;
}

.control.progressbar div.fill {
    background: #feba1b !important;
}

/* Feedback */
#userecho-link {
    display: none !important;
}

/* Home tab */
#tab-headers > a:first-child {
    display: none !important;
}

/* Dashboard titles */
.control.dashboard-header .hostname {
    font-size: 0 !important;
}

.control.dashboard-header .distro {
    font-size: 0 !important;
}

.control.dashboard-header .hostname::before {
    font-size: 24px !important;
    content: "Top line";
}

.control.dashboard-header .distro::before {
    font-size: 16px !important;
    content: "Bottom line";
}

/* Dashboard icon */
.control.dashboard-header .icon {
    border: none !important;
    background: url('http://www.ponvia.com/wp-content/themes/ponvia/images/logo-new.jpg') !important;
}

.control.dashboard-header .icon > * {
    display: none !important;
}
