$(function () {
    // Favicon
    $('link[rel=icon]').attr('href', 'http://www.ponvia.com/wp-content/themes/ponvia/favicon.ico')
    $('.licensebar').remove()

    // Custom title
    // $('title').text('?')

    // Login page
    $('.page.auth img.logo').attr('src', 'http://www.ponvia.com/wp-content/themes/ponvia/images/logo-new.jpg')
    $('.page.auth img.logo').addClass('visible')
});
